"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shuffle = require("lodash.shuffle");
const constants_1 = require("../constants");
const _removeCommonIPsFromLists = (peerList) => {
    const peerMap = new Map();
    for (const peer of peerList) {
        const { sharedState } = peer;
        const peerHeight = sharedState && sharedState.height ? sharedState.height : 0;
        const tempPeer = peerMap.get(peer.ipAddress);
        if (tempPeer) {
            const { sharedState: tempSharedState } = tempPeer;
            const tempPeerHeight = tempSharedState && tempSharedState.height
                ? tempSharedState.height
                : 0;
            if (peerHeight > tempPeerHeight) {
                peerMap.set(peer.ipAddress, peer);
            }
        }
        else {
            peerMap.set(peer.ipAddress, peer);
        }
    }
    return [...peerMap.values()];
};
exports.selectPeersForRequest = (input) => {
    const { peers } = input;
    const peerLimit = input.peerLimit;
    if (peers.length === 0) {
        return [];
    }
    if (peerLimit === undefined) {
        return shuffle(peers);
    }
    return shuffle(peers).slice(0, peerLimit);
};
exports.selectPeersForSend = (input) => {
    const shuffledPeers = shuffle(input.peers);
    const peerLimit = input.peerLimit;
    const halfPeerLimit = Math.round(peerLimit / 2);
    const outboundPeers = shuffledPeers.filter((peerInfo) => peerInfo.internalState
        ? peerInfo.internalState.connectionKind === constants_1.ConnectionKind.OUTBOUND
        : false);
    const inboundPeers = shuffledPeers.filter((peerInfo) => peerInfo.internalState
        ? peerInfo.internalState.connectionKind === constants_1.ConnectionKind.INBOUND
        : false);
    let shortestPeersList;
    let longestPeersList;
    if (outboundPeers.length < inboundPeers.length) {
        shortestPeersList = outboundPeers;
        longestPeersList = inboundPeers;
    }
    else {
        shortestPeersList = inboundPeers;
        longestPeersList = outboundPeers;
    }
    const selectedFirstKindPeers = shortestPeersList.slice(0, halfPeerLimit);
    const remainingPeerLimit = peerLimit - selectedFirstKindPeers.length;
    const selectedSecondKindPeers = longestPeersList.slice(0, remainingPeerLimit);
    return selectedFirstKindPeers.concat(selectedSecondKindPeers);
};
exports.selectPeersForConnection = (input) => {
    if ((input.peerLimit && input.peerLimit < 0) ||
        (input.triedPeers.length === 0 && input.newPeers.length === 0)) {
        return [];
    }
    if (input.peerLimit === undefined ||
        input.peerLimit >= input.triedPeers.length + input.newPeers.length) {
        return _removeCommonIPsFromLists([...input.newPeers, ...input.triedPeers]);
    }
    const minimumProbability = 0.5;
    const x = input.triedPeers.length < 100
        ? minimumProbability
        : input.triedPeers.length /
            (input.triedPeers.length + input.newPeers.length);
    const r = Math.max(x, minimumProbability);
    const shuffledTriedPeers = shuffle(input.triedPeers);
    const shuffledNewPeers = shuffle(input.newPeers);
    const peerList = [...Array(input.peerLimit)].map(() => {
        if (shuffledTriedPeers.length !== 0) {
            if (Math.random() < r) {
                return shuffledTriedPeers.pop();
            }
        }
        if (shuffledNewPeers.length !== 0) {
            return shuffledNewPeers.pop();
        }
        return shuffledTriedPeers.pop();
    });
    return _removeCommonIPsFromLists(peerList);
};
//# sourceMappingURL=select.js.map